using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Kerr.WindowClippings;

namespace Kerr.WindowClippings.FilterWatermark
{
    [Guid("AEF24476-A34F-4590-8377-01E0A847347B")]
    public class FilterWatermark : Filter
    {
        /// <summary>
        /// The constructor sets the add-in defaults.
        /// </summary>
        public FilterWatermark()
        {
            Name = "Add watermark";
            HasSettings = true;
            m_alignment = AnchorStyles.Bottom | AnchorStyles.Right;
        }

        /// <summary>
        /// Loads the persisted image path and alignment.
        /// </summary>
        protected override void LoadSettings(BinaryReader reader)
        {
            m_imagePath = reader.ReadString();
            m_alignment = (AnchorStyles) reader.ReadUInt32();
        }

        /// <summary>
        /// Saves the image path and alignment.
        /// </summary>
        protected override void SaveSettings(BinaryWriter writer)
        {
            writer.Write(m_imagePath);
            writer.Write((uint) m_alignment);
        }

        /// <summary>
        /// Show the settings window.
        /// </summary>
        protected override void EditSettings(IWin32Window parentWindow)
        {
            using (FilterWatermarkWindow window = new FilterWatermarkWindow())
            {
                window.ImagePath = m_imagePath;
                window.Alignment = m_alignment;

                if (DialogResult.OK == window.ShowDialog(parentWindow))
                {
                    m_imagePath = window.ImagePath;
                    m_alignment = window.Alignment;
                }
            }
        }

        /// <summary>
        /// Adds the watermark to the given bitmap.
        /// </summary>
        protected override void Process(Bitmap bitmap)
        {
            using (Bitmap watermark = new Bitmap(m_imagePath))
            using (Graphics destination = Graphics.FromImage(bitmap))
            {
                Helpers.DrawWatermark(destination,
                                      bitmap.Width,
                                      bitmap.Height,
                                      watermark,
                                      m_alignment);
            }
        }

        private string m_imagePath;
        private AnchorStyles m_alignment;
    }
}
